@@ -1,7 +1,8 @@
-use crate::jwt;
 use anyhow::{bail, Context, Result};
+use base64::Engine;
 use sccache::dist::http::{ClientAuthCheck, ClientVisibleMsg};
 use sccache::util::{new_reqwest_blocking_client, BASE64_URL_SAFE_ENGINE};
+use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
 use std::result::Result as StdResult;
 use std::sync::Mutex;
@@ -29,9 +30,11 @@ impl Jwk {
         }
 
         // JWK is big-endian, openssl bignum from_slice is big-endian
-        let n = base64::decode_engine(&self.n, &BASE64_URL_SAFE_ENGINE)
+        let n = BASE64_URL_SAFE_ENGINE
+            .decode(&self.n)
             .context("Failed to base64 decode n")?;
-        let e = base64::decode_engine(&self.e, &BASE64_URL_SAFE_ENGINE)
+        let e = BASE64_URL_SAFE_ENGINE
+            .decode(&self.e)
             .context("Failed to base64 decode e")?;
         let n_bn = openssl::bn::BigNum::from_slice(&n)
             .context("Failed to create openssl bignum from n")?;
@@ -165,7 +168,7 @@ impl MozillaCheck {
         }
 
         // The API didn't return a HTTP error code, let's check the response
-        let () = check_mozilla_profile(&user, &self.required_groups, &res_text)
+        check_mozilla_profile(&user, &self.required_groups, &res_text)
             .with_context(|| format!("Validation of the user profile failed for {}", user))?;
 
         // Validation success, cache the token
