@@ -15,6 +15,7 @@
 #[macro_use]
 mod args;
 mod c;
+mod cicc;
 mod clang;
 #[macro_use]
 #[allow(clippy::module_inception)]
@@ -25,10 +26,12 @@ mod msvc;
 mod nvcc;
 mod nvhpc;
 mod preprocessor_cache;
+mod ptxas;
 mod rust;
 mod tasking_vx;
 #[macro_use]
 mod counted_array;
 
+pub use crate::compiler::c::CCompilerKind;
 pub use crate::compiler::compiler::*;
 pub use crate::compiler::preprocessor_cache::PreprocessorCacheEntry;
